/*
 *  Copyright (C) 2002 Matthew Colyer (linuxcoder@colyer.org)
 *  Copyright (C) 2002 Anatoly Asviyan (aka Arsen)
 *	Copyright (C) 2000 Peter Zelezny
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdlib.h>
#include <locale.h>
#include <sys/types.h>

#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/Xutil.h>
#include <X11/xpm.h>
#include <X11/Xatom.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "wm.h"
#include "panel.h"
#include "support.h"
#include "launchbar.h"
#include "taskbar.h"
#include "menu.h"
#include "clock.h"



char *atom_names[] = {
	"KWM_WIN_ICON",
	"_MOTIF_WM_HINTS",
	"_WIN_HINTS",
	"_WIN_LAYER",
	"_WIN_STATE",
	"WM_STATE",

	"_NET_CLIENT_LIST",
	"_NET_NUMBER_OF_DESKTOPS",
	"_NET_CURRENT_DESKTOP",
	"_NET_DESKTOP_NAMES",
	"_NET_ACTIVE_WINDOW",

	"_NET_WM_STATE",
	"_NET_WM_STATE_SKIP_TASKBAR",
	"_NET_WM_STATE_SKIP_PAGER",
	"_NET_WM_STATE_STICKY",
	"_NET_WM_STATE_SHADED",

	"_NET_WM_DESKTOP",
	"_NET_WM_NAME",

	"_WIN_WORKSPACE_NAMES",
};

#define ATOM_COUNT (sizeof (atom_names) / sizeof (atom_names[0]))

Atom atoms[ATOM_COUNT];

GtkWidget *window, *hbox;

Window topxwin;


void show_menu(GtkWidget * widget, gpointer menu);
void set_menu_position(GtkMenu * menu, gint * x, gint * y, gpointer data);
gint delete_event(GtkWidget *, GdkEvent *, gpointer);
gint destroy_event(GtkWidget *, GdkEvent *, gpointer);



void *get_xaproperty(Window win, Atom prop, Atom type, int *nitems)
{
	Atom type_ret;
	int format_ret;
	unsigned long items_ret;
	unsigned long after_ret;
	unsigned char *prop_data;

	prop_data = 0;

	XGetWindowProperty(GDK_DISPLAY(), win, prop, 0, 0x7fffffff, False,
			   type, &type_ret, &format_ret, &items_ret,
			   &after_ret, &prop_data);
	if (nitems)
		*nitems = items_ret;
	/* must be freed with XFree(prop_data) */
	return prop_data;
}


char **get_textproperty(Window win, Atom prop, int *nitems)
{
	XTextProperty text_prop;
	char **list;
	int ret;

	if (XGetTextProperty(GDK_DISPLAY(), win, &text_prop, prop)) {
		ret = XmbTextPropertyToTextList(GDK_DISPLAY(), &text_prop,
						&list, nitems);
		XFree(text_prop.value);
		if ((ret == Success) && (*nitems > 0) && *list) {
			/* must be freed with XFreeStringList(list); */
			return list;
		}
	}
	return NULL;
}


void panel_props()
{
	Atom state[3];
	unsigned int val;
	GtkRequisition requisition;

	Atom real_type;
	int temp, real_format;
	unsigned long items_read, items_left;
	long *data = NULL;
	Atom net_wm_strut;
	CARD32 strut[] = { 0, 0, 0, PANEL_HEIGHT + 1 };
	
	gtk_widget_size_request(window, &requisition);
	gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
	gtk_widget_set_usize(window, gdk_screen_width(), PANEL_HEIGHT);
	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);

	net_wm_strut = XInternAtom (GDK_DISPLAY(), "GNOME_PANEL_DESKTOP_AREA", False);
	
	XChangeProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(), net_wm_strut, XA_CARDINAL, 32,
		   PropModeReplace, (unsigned char *) &strut, 4);
	  
	state[0] = atom__NET_WM_STATE_STICKY;
	state[1] = atom__NET_WM_STATE_SKIP_PAGER;
	state[2] = atom__NET_WM_STATE_SKIP_TASKBAR;

	XChangeProperty(GDK_DISPLAY(), topxwin, atom__NET_WM_STATE,
			XA_ATOM, 32, PropModeReplace,
			(unsigned char *) state, 3);

	val = WIN_STATE_STICKY | WIN_STATE_FIXED_POSITION;
	XChangeProperty(GDK_DISPLAY(), topxwin, atom__WIN_STATE,
			XA_CARDINAL, 32, PropModeReplace,
			(unsigned char *) &val, 1);

	val = WIN_HINTS_SKIP_FOCUS | WIN_HINTS_SKIP_WINLIST |
	    WIN_HINTS_SKIP_TASKBAR;
	XChangeProperty(GDK_DISPLAY(), topxwin, atom__WIN_HINTS,
			XA_CARDINAL, 32, PropModeReplace,
			(unsigned char *) &val, 1);


	val = WIN_LAYER_DOCK;
	XChangeProperty(GDK_DISPLAY(), topxwin, atom__WIN_LAYER,
			XA_CARDINAL, 32, PropModeReplace,
			(unsigned char *) &val, 1);

}


void panel_pos()
{
	GtkRequisition requisition;

	gtk_widget_size_request(window, &requisition);
	gdk_window_move_resize(window->window, 0,
			       gdk_screen_height() - requisition.height,
			       gdk_screen_width(), requisition.height);
}


int main(int argc, char *argv[])
{
	GtkWidget *ret, *sep, *root_menu, *root_menu_label;
	gchar *launch_icon_filename;

	setlocale(LC_CTYPE, "");
	gtk_set_locale();
	gtk_init(&argc, &argv);
	XSetLocaleModifiers("");
	XInternAtoms(GDK_DISPLAY(), atom_names, ATOM_COUNT, False, atoms);

	/*create and size the panel window */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 0);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			   GTK_SIGNAL_FUNC(delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
			   GTK_SIGNAL_FUNC(destroy_event), NULL);
	gtk_widget_realize(window);
	topxwin = GDK_WINDOW_XWINDOW(GTK_WIDGET(window)->window);

	/*create container to hold the widgets on the panel */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), hbox);

	/*create, load and attach the root menu */
	if (!(root_menu = panel_menu_load())) {
		g_error("Panel menu unable to load: Serious Error\n");
	}
	ret = gtk_button_new();
	gtk_container_set_border_width(GTK_CONTAINER(ret), 0);
	
	gtk_signal_connect(GTK_OBJECT(ret), "clicked",
			   GTK_SIGNAL_FUNC(show_menu), root_menu);
	launch_icon_filename =
	    g_build_filename(PKGIMAGEDIR, LAUNCH_ICON_FILE, NULL);
	root_menu_label = GTK_WIDGET(icon_label(launch_icon_filename));
	g_free(launch_icon_filename);
	gtk_widget_show(root_menu_label);
	gtk_container_add(GTK_CONTAINER(ret), root_menu_label);
	gtk_button_set_relief(GTK_BUTTON(ret), GTK_RELIEF_NONE);
	gtk_box_pack_start(GTK_BOX(hbox), ret, FALSE, FALSE, 0);
	
	/*add a vertical seperator */
	sep = gtk_vseparator_new();
	gtk_box_pack_start(GTK_BOX(hbox), sep, FALSE, FALSE, 0);

	/*create, load and attach the launchbar */
	ret = panel_launchbar_load();
	if (ret != NULL) {
		/*if the launchbar file is empty contiune */
		gtk_box_pack_start(GTK_BOX(hbox), ret, FALSE, FALSE, 0);
	}

	/*add a vertical seperator */
	sep = gtk_vseparator_new();
	gtk_box_pack_start(GTK_BOX(hbox), sep, FALSE, FALSE, 0);

	/*add a vertical seperator */
	sep = gtk_vseparator_new();
	gtk_box_pack_start(GTK_BOX(hbox), sep, FALSE, FALSE, 0);

	/*create, load and attach the taskbar */
	ret = tb_init();
	gtk_box_pack_start(GTK_BOX(hbox), ret, TRUE, TRUE, 0);

	/*create, load and attach the clock */
	ret = clock_init();
	gtk_box_pack_end(GTK_BOX(hbox), ret, FALSE, FALSE, 0);
	sep = gtk_vseparator_new();
	gtk_box_pack_end(GTK_BOX(hbox), sep, FALSE, FALSE, 0);


	gtk_widget_show_all(hbox);

	panel_props();

	gtk_widget_show_all(window);

	panel_pos();

	gtk_main();
	return 0;
}

void show_menu(GtkWidget * widget, gpointer menu)
{
	gtk_menu_popup(menu, NULL, NULL,
		       (GtkMenuPositionFunc) set_menu_position, NULL, 0,
		       0);
}

void set_menu_position(GtkMenu * menu, gint * x, gint * y, gpointer data)
{

	gint screen_width;
	gint screen_height;
	int screen_basex, screen_basey;
	GtkRequisition requisition;

	gtk_widget_get_child_requisition(GTK_WIDGET(menu), &requisition);

	screen_width = gdk_screen_width();
	screen_height = gdk_screen_height();

	*x = 0;
	*y = (screen_height - PANEL_HEIGHT) - requisition.height;
}


gint delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
	return FALSE;
}

gint destroy_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
	gtk_main_quit();
	return FALSE;
}
